
export default class WebCamera {

    private id: string
    private config: any
    private video?: HTMLVideoElement
    private canvas?: HTMLCanvasElement
    private isInit = false
    private isOpen = false

    constructor(id: string, config: any) {

        if (!config || !config.width || !config.height)
            throw Error(`constructor|配置参数错误: ${this.config}`)

        if (!config.background)
            config.background = "#fff"

        this.id = id
        this.config = config
    }

    private init() {

        if (this.isInit) {
            console.warn("Camera已经初始化");
            return
        }

        this.isInit = false
        this.isOpen = false

        const mount = document.getElementById(this.id)
        if (!mount) throw Error(`init|找不到具有指定id的元素: ${this.id}`)

        if (!this.config || typeof this.config !== "object") throw Error(`init|配置参数错误: ${this.config}`)

        const { width, height, background } = this.config;

        mount.style['display'] = 'flex'
        mount.style['justify-content'] = 'center'
        mount.style['align-items'] = 'center'
        mount.style['position'] = 'relative'
        mount.style['overflow'] = 'auto';

        ['canvas', 'video'].forEach(tagName => {

            const element = document.createElement(tagName)
            element.setAttribute("width", width)
            element.setAttribute("height", height)
            element.style.width = `${width}px`
            element.style.height = `${height}px`
            element.style.position = "absolute"
            element.style.background = background

            mount.appendChild(element)
        })

        this.video = mount.getElementsByTagName('video')[0]
        this.canvas = mount.getElementsByTagName('canvas')[0]

        this.video.style.zIndex = "100"
        this.canvas.style.zIndex = "1"
        mount.style.zIndex = "102"

        this.isInit = true
    }

    public start() {
        this.init()
        return this.open()
    }

    private open() {
        return new Promise((resolve, reject) => {

            if (this.isOpen)
                resolve("摄像头以及开启")

            if (!document.location.protocol.startsWith('https'))
                reject('非https协议无法打开摄像头')

            const { width, height, audio, isFront } = this.config;
            let constraints = {
                video: {            //这里是摄像头的信息
                    height: width,
                    width: height,
                    facingMode: isFront ? 'user' : 'environment'
                },
                audio: audio || false,  //是否开启麦克风
            }
            navigator.mediaDevices.getUserMedia(constraints) //这里主要是用于请求用户打开摄像头的权限
                .then(stream => {     //可以看出是使用promise封装的 那么我们就可以使用then和catch
                    this.video!.srcObject = stream;    //用户允许时 将摄像头对象的画面转移到video上面
                    this.video!.play();                //打开video的画面
                    this.isOpen = true
                    resolve("摄像头开启成功")
                }).catch((err) => {
                    reject(err)
                })
        })
    }

    public capture() {

        const { width, height } = this.config;
        this.canvas!.getContext("2d")!.drawImage(this.video!, 0, 0, width, height);   //第一个参数为要截取的dom对象，第2个和第3个为xy轴的偏移值,3-4为截取图像的大小
        const dataURL = this.canvas!.toDataURL('image/jpeg')
        return new Promise((resolve, reject) => {
            return resolve(dataURL)
        })
    }

    public stop() {

        //这里如果打开了麦克风、getTracks是一个数组，我们同样需要获取下标[1]来关闭摄像头 打开麦克风[0]就是麦克风
        const tracks = (this.video!.srcObject! as any).getTracks();
        for (let i = 0; i < tracks.length; i++) {
            tracks[i].stop();
        }
    }

}

